<?

/*

 	Webmedia Explorer 6.0 - Multimedia Management System
 	Copyright (C) 2009 Marc Salmurri (marc.salmurri@gmail.com)

 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
 	the Free Software Foundation; either version 2 of the License, or (at
 	your option) any later version.

 	This program is distributed in the hope that it will be useful, but
 	WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 	General Public License for more details.

 	You should have received a copy of the GNU General Public License
 	along with this program; if not, write to the Free Software
 	Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA  02110-1301, USA.

*/

class zip
{
	var $filename;
	var $overwrite;
	
	var $zip_signature 		= "\x50\x4b\x03\x04"; 		// local file header signature
	var $dir_signature 		= "\x50\x4b\x01\x02"; 		// central dir header signature
	var $dir_signatureE 		= "\x50\x4b\x05\x06"; 		// end of central dir signature

	var $files_count  		= 0;
	var $file_handler;
	
	function zip ( $filename, $overwrite = true )
	{
		$this->filename  	= $filename;
		$this->overwrite 	= $overwrite;
	}

	function add_dir ( $dirname, $file_comments = '' )
	{
		if ( substr ( $dirname, -1 ) != '/' ) $dirname .= '/';

		$this->add_file ( false, $dirname, $file_comments );
	}
	
	function add_file ( $filename, $cfilename, $file_comments = '', $data = false )
	{
		if ( !( $file_handler = &$this->file_handler ) )	

			$file_handler = fopen ( $GLOBALS["path_zip"].$this->filename, $this->overwrite ? 'wb' : 'a+b' );

		if ( substr ( $cfilename, -1 ) == '/' )							// $filename can be a local file OR the data wich will be compressed
		{
			$details['uncsize'] 		= 0;
			$data 				= '';
		}
		elseif ( file_exists ( $filename ) )
		{
			$details['uncsize'] 		= filesize ( $filename );
			$data 				= file_get_contents ( $filename );
		}
		elseif ( $filename )
		{
			return false;
		}
		else{
			$details['uncsize'] 		= strlen ( $filename );
			// DATA is given.. use it! :|
		}
		if ( $details['uncsize'] < 256 )								// if data to compress is too small, just store it
		{
			$details['comsize']		= $details['uncsize'];
			$details['vneeded'] 		= 10;
			$details['cmethod'] 		= 0;
			$zdata 				= &$data;
		}
		else													// otherwise, compress it
		{
			$zdata				= gzcompress ( $data );
			$zdata 				= substr ( substr ( $zdata, 0, strlen ( $zdata ) - 4 ), 2 ); 
			$details['comsize'] 		= strlen ( $zdata );
			$details['vneeded'] 		= 10;
			$details['cmethod'] 		= 8;
		}
		
		$details['bitflag'] 			= 0;
		$details['crc_32']  			= crc32 ( $data );
		
		// Convert date and time to DOS Format, and set then

		$lastmod_timeS  			= str_pad ( decbin ( date ( 's' ) >= 32 ? date ( 's' ) - 32 : date ( 's' ) ), 5, '0', STR_PAD_LEFT );
		$lastmod_timeM  			= str_pad ( decbin ( date ( 'i' ) ), 6, '0', STR_PAD_LEFT );
		$lastmod_timeH  			= str_pad ( decbin ( date ( 'H' ) ), 5, '0', STR_PAD_LEFT );
		$lastmod_dateD 			= str_pad ( decbin ( date ( 'd' ) ), 5, '0', STR_PAD_LEFT );
		$lastmod_dateM  			= str_pad ( decbin ( date ( 'm' ) ), 4, '0', STR_PAD_LEFT );
		$lastmod_dateY  			= str_pad ( decbin ( date ( 'Y' ) - 1980 ), 7, '0', STR_PAD_LEFT );

		$details['modtime'] 			= bindec ( "$lastmod_timeH$lastmod_timeM$lastmod_timeS" );
		$details['moddate'] 			= bindec ( "$lastmod_dateY$lastmod_dateM$lastmod_dateD" );

		$details['offset'] 			= ftell ( $file_handler );

		fwrite ( $file_handler, $this->zip_signature );

		fwrite ( $file_handler, pack ('s', $details['vneeded'] ) ); 	// version_needed
		fwrite ( $file_handler, pack ( 's', $details['bitflag'] ) ); 	// general_bit_flag
		fwrite ( $file_handler, pack ( 's', $details['cmethod'] ) ); 	// compression_method
		fwrite ( $file_handler, pack ( 's', $details['modtime'] ) ); 	// lastmod_time
		fwrite ( $file_handler, pack ( 's', $details['moddate'] ) ); 	// lastmod_date
		fwrite ( $file_handler, pack ( 'V', $details['crc_32'] ) );  	// crc-32
		fwrite ( $file_handler, pack ( 'I', $details['comsize'] ) ); 	// compressed_size
		fwrite ( $file_handler, pack ( 'I', $details['uncsize'] ) ); 	// uncompressed_size
		fwrite ( $file_handler, pack ( 's', strlen ( $cfilename ) ) );   // file_name_length
		fwrite ( $file_handler, pack ( 's', 0 ) );  				// extra_field_length
		fwrite ( $file_handler, $cfilename );    				// file_name

		// ignoring extra_field

		fwrite ( $file_handler, $zdata );
		
		// Append it to central dir

		$details['external_attributes']  	= ( substr ( $cfilename, -1 )== '/' && !$zdata ) ? 16 : 32; 		// Directory or file name
		$details['comments']             	= $file_comments;

		$this->append_central_dir ( $cfilename, $details );
		$this->files_count++;
	}

	function set_extra ( $filename, $property, $value )
	{
		$this->centraldirs[$filename][$property] = $value;
	}

	function close ( $zip_comments = '' )
	{
		if ( !( $file_handler = &$this->file_handler ) )
		
			$file_handler = fopen ( $GLOBALS["path_zip"].$this->filename, $this->overwrite ? 'w' : 'a+' );
		
		$cdrec = "";

		foreach ( $this->centraldirs as $filename => $cd )
		{
			$cdrec .= $this->dir_signature;
			$cdrec .= "\x0\x0";                 							// version made by
			$cdrec .= pack ( 'v', $cd['vneeded'] ); 					// version needed to extract
			$cdrec .= "\x0\x0";                  							// general bit flag
			$cdrec .= pack ( 'v', $cd['cmethod'] ); 					// compression method
			$cdrec .= pack ( 'v', $cd['modtime'] ); 					// lastmod time
			$cdrec .= pack ( 'v', $cd['moddate'] ); 					// lastmod date
			$cdrec .= pack ( 'V', $cd['crc_32'] );  					// crc32
			$cdrec .= pack ( 'V', $cd['comsize'] ); 					// compressed filesize
			$cdrec .= pack ( 'V', $cd['uncsize'] ); 					// uncompressed filesize
			$cdrec .= pack ( 'v', strlen ( $filename ) ); 				// file comment length
			$cdrec .= pack ( 'v', 0 );                						// extra field length
			$cdrec .= pack ( 'v', strlen ( $cd['comments'] ) );			// file comment length
			$cdrec .= pack ( 'v', 0 ); 								// disk number start
			$cdrec .= pack ( 'v', 0 ); 								// internal file attributes
			$cdrec .= pack ( 'V', $cd['external_attributes'] ); 			// internal file attributes
			$cdrec .= pack ( 'V', $cd['offset'] ); 						// relative offset of local header
			$cdrec .= $filename;
			$cdrec .= $cd['comments'];
		}

		$before_cd = ftell ( $file_handler );

		fwrite ( $file_handler, $cdrec );
		
		// end of central dir

		fwrite ( $file_handler, $this->dir_signatureE );
		fwrite ( $file_handler, pack ( 'v', 0) ); 						// number of this disk
		fwrite ( $file_handler, pack ( 'v', 0 ) ); 						// number of the disk with the start of the central directory
		fwrite ( $file_handler, pack ( 'v', $this->files_count ) ); 			// total # of entries "on this disk" 
		fwrite ( $file_handler, pack ( 'v', $this->files_count ) ); 			// total # of entries overall 
		fwrite ( $file_handler, pack ( 'V', strlen ( $cdrec ) ) );     			// size of central dir 
		fwrite ( $file_handler, pack ( 'V', $before_cd ) );         			// offset to start of central dir
		fwrite ( $file_handler, pack ( 'v', strlen ( $zip_comments ) ) ); 		// .zip file comment length

		fwrite ( $file_handler, $zip_comments );

		fclose ( $file_handler );
	}

	function append_central_dir ( $filename, $properties )
	{
		$this->centraldirs[$filename] = $properties;
	}
}

?>